@@ -608,9 +608,12 @@ class DataTransform(forms.ModelForm):
         if 'data' in kwargs.keys():
             query_dict = kwargs['data'].dict()
             form_data = {}
-            form_data['id'] = query_dict['id']
-            form_data['accessory'] = query_dict['accessory']
-            form_data['type'] = query_dict['type']
+            if 'id' in query_dict:
+                form_data['id'] = query_dict['id']
+            if 'accessory' in query_dict:
+                form_data['accessory'] = query_dict['accessory']
+            if 'type' in query_dict:
+                form_data['type'] = query_dict['type']
 
             nested_data_transformer_form_data = get_values_for_keys_containing(nested_data_transformer_form_prefix,
                                                                                query_dict)
@@ -628,7 +631,7 @@ class DataTransform(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
+                    initial=[{'parent': str(self.instance.id),
                               'id': str(uuid.uuid4())}],
                     prefix=nested_data_transformer_form_prefix,
                 )
@@ -641,7 +644,7 @@ class DataTransform(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),
+                    initial=[{'parent': str(self.instance.id),
                               'id': str(uuid.uuid4())}],
                 )
                 self.nested_data_transformer_form.extra = 0
@@ -653,7 +656,7 @@ class DataTransform(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),
+                    initial=[{'parent': str(self.instance.id),
                               'id': str(uuid.uuid4())}],
                 )
                 self.nested_data_transformer_form.extra = 0
@@ -664,7 +667,7 @@ class DataTransform(forms.ModelForm):
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
             prefix=nested_constants_form_prefix,
-            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
+            initial=[{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())}]
 
         )
         self.nested_constants_form.extra = self.instance.data_transformer_constants.count()
@@ -774,6 +777,10 @@ class DataTransformRoot(forms.ModelForm):
     #     super(HardwareTypeForm, self).__init__(*args, **kwargs)
     def __init__(self, *args, **kwargs):
         super(DataTransformRoot, self).__init__(*args, **kwargs)
+        # if hasattr(self.instance, 'id'):
+        #     ins_id = str(self.instance.id)
+        # else:
+        #     ins_id = str(self.instance.id)
         nested_data_transformer_form_prefix = str(self.instance.id) + 'dataTransformerInlineFormset'
         nested_constants_form_prefix = str(self.instance.id) + 'DataConstantsTransformerInlineFormset'
         nested_hardware_outputs_form_prefix = str(self.instance.id) + 'HardwareTransformerForm'
@@ -785,11 +792,15 @@ class DataTransformRoot(forms.ModelForm):
         nested_hardware_stats_form_data = None
 
         if 'data' in kwargs.keys():
-            query_dict = kwargs['data'].dict()
-            form_data = {}
-            form_data['id'] = query_dict['id']
-            form_data['accessory'] = query_dict['accessory']
-            form_data['type'] = query_dict['type']
+            if 'data' in kwargs.keys():
+                query_dict = kwargs['data'].dict()
+                form_data = {}
+                if 'id' in query_dict:
+                    form_data['id'] = query_dict['id']
+                if 'accessory' in query_dict:
+                    form_data['accessory'] = query_dict['accessory']
+                if 'type' in query_dict:
+                    form_data['type'] = query_dict['type']
 
             nested_data_transformer_form_data = get_values_for_keys_containing(nested_data_transformer_form_prefix,
                                                                                query_dict)
@@ -825,7 +836,7 @@ class DataTransformRoot(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),
+                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id),
                               'id': str(uuid.uuid4())}],
                 )
                 self.nested_data_transformer_form.extra = 0
@@ -837,7 +848,7 @@ class DataTransformRoot(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),
+                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id),
                               'id': str(uuid.uuid4())}],
                 )
                 self.nested_data_transformer_form.extra = 0
@@ -849,7 +860,7 @@ class DataTransformRoot(forms.ModelForm):
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
             prefix=nested_constants_form_prefix,
-            initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
+            initial=[{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())}]
 
         )
         self.nested_constants_form.extra = self.instance.data_transformer_constants.count()
@@ -932,5 +943,5 @@ class DataTransformRoot(forms.ModelForm):
 
     class Meta:
         model = DataTransformer
-        fields = ['id', 'accessory', 'type']
+        fields = ['id', 'accessory', 'type', 'channels','channel_stats']
 # DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
